@@ -5,7 +5,7 @@ use std::{
 
 /// The node's configuration.
 #[derive(Debug, Clone)]
-pub struct NodeConfig {
+pub struct Config {
     /// The name/identifier of the node.
     ///
     /// note: if set to `None`, the `Node` will automatically be assigned a sequential, zero-based numeric identifier.
@@ -20,14 +20,6 @@ pub struct NodeConfig {
     pub allow_random_port: bool,
     /// The depth of the queues passing connections to protocol handlers.
     pub protocol_handler_queue_depth: usize,
-    /// The size of a per-connection buffer for reading inbound messages.
-    pub conn_read_buffer_size: usize,
-    /// The size of a per-connection buffer for writing outbound messages.
-    pub conn_write_buffer_size: usize,
-    /// The depth of per-connection queues used to process inbound messages.
-    pub conn_inbound_queue_depth: usize,
-    /// The depth of per-connection queues used to send outbound messages.
-    pub conn_outbound_queue_depth: usize,
     /// The delay on the next read attempt from a connection that can't be read from.
     pub invalid_read_delay_secs: u64,
     /// The list of IO errors considered fatal and causing the connection to be dropped.
@@ -37,11 +29,18 @@ pub struct NodeConfig {
     /// note: this number can very briefly be breached by 1 in case of inbound connection attempts. It can never be
     /// breached by outbound connection attempts, though.
     pub max_connections: u16,
+
+    /// The size of a per-connection buffer for reading inbound messages.
+    pub read_buffer_size: usize,
+    /// The depth of per-connection queues used to process inbound messages.
+    pub inbound_queue_depth: usize,
+    /// The depth of per-connection queues used to send outbound messages.
+    pub outbound_queue_depth: usize,
     /// The maximum time allowed for a connection to perform a handshake before it is rejected.
     pub max_handshake_time_ms: u64,
 }
 
-impl Default for NodeConfig {
+impl Default for Config {
     fn default() -> Self {
         #[cfg(feature = "test")]
         fn default_ip() -> Option<IpAddr> {
@@ -59,10 +58,6 @@ impl Default for NodeConfig {
             desired_listening_port: None,
             allow_random_port: true,
             protocol_handler_queue_depth: 16,
-            conn_read_buffer_size: 64 * 1024,
-            conn_write_buffer_size: 64 * 1024,
-            conn_inbound_queue_depth: 64,
-            conn_outbound_queue_depth: 64,
             invalid_read_delay_secs: 10,
             fatal_io_errors: vec![
                 ConnectionReset,
@@ -72,6 +67,10 @@ impl Default for NodeConfig {
                 UnexpectedEof,
             ],
             max_connections: 100,
+
+            read_buffer_size: 64 * 1024,
+            inbound_queue_depth: 64,
+            outbound_queue_depth: 64,
             max_handshake_time_ms: 3_000,
         }
     }
