@@ -1,9 +1,6 @@
 //! Objects associated with connection handling.
 
-use crate::Node;
-
 use bytes::Bytes;
-use fxhash::FxHashMap;
 use parking_lot::RwLock;
 use tokio::{
     net::{
@@ -13,12 +10,11 @@ use tokio::{
     sync::mpsc::Sender,
     task::JoinHandle,
 };
-use tracing::*;
 
-use std::{io, net::SocketAddr, ops::Not};
+use std::{collections::HashMap, io, net::SocketAddr, ops::Not};
 
 #[derive(Default)]
-pub(crate) struct Connections(RwLock<FxHashMap<SocketAddr, Connection>>);
+pub(crate) struct Connections(RwLock<HashMap<SocketAddr, Connection>>);
 
 impl Connections {
     pub(crate) fn sender(&self, addr: SocketAddr) -> io::Result<Sender<Bytes>> {
@@ -45,8 +41,8 @@ impl Connections {
         self.0.read().contains_key(&addr)
     }
 
-    pub(crate) fn remove(&self, addr: SocketAddr) -> bool {
-        self.0.write().remove(&addr).is_some()
+    pub(crate) fn remove(&self, addr: SocketAddr) -> Option<Connection> {
+        self.0.write().remove(&addr)
     }
 
     pub(crate) fn num_connected(&self) -> usize {
@@ -81,8 +77,6 @@ impl Not for ConnectionSide {
 /// Keeps track of tasks that have been spawned for the purposes of a connection; it
 /// also contains a sender that communicates with the `Writing` protocol handler.
 pub struct Connection {
-    /// A reference to the owning node.
-    pub node: Node,
     /// The address of the connection.
     pub addr: SocketAddr,
     /// Kept only until the protocols are enabled (`Reading` should `take()` it).
@@ -99,16 +93,10 @@ pub struct Connection {
 
 impl Connection {
     /// Creates a `Connection` with placeholders for protocol-related objects.
-    pub(crate) fn new(
-        addr: SocketAddr,
-        stream: TcpStream,
-        side: ConnectionSide,
-        node: &Node,
-    ) -> Self {
+    pub(crate) fn new(addr: SocketAddr, stream: TcpStream, side: ConnectionSide) -> Self {
         let (reader, writer) = stream.into_split();
 
         Self {
-            node: node.clone(),
             addr,
             reader: Some(reader),
             writer: Some(writer),
@@ -137,26 +125,7 @@ impl Connection {
         if let Some(ref sender) = self.outbound_message_sender {
             Ok(sender.clone())
         } else {
-            error!(parent: self.node.span(), "can't send messages: the Writing protocol is disabled");
             Err(io::ErrorKind::Unsupported.into())
         }
     }
 }
-
-impl Drop for Connection {
-    fn drop(&mut self) {
-        debug!(parent: self.node.span(), "disconnecting from {}", self.addr);
-
-        // shut the associated tasks down
-        for task in self.tasks.iter().rev() {
-            task.abort();
-        }
-
-        // if the (owning) node was not the initiator of the connection, it doesn't know the listening address
-        // of the associated peer, so the related stats are unreliable; the next connection initiated by the
-        // peer could be bound to an entirely different port number
-        if matches!(self.side, ConnectionSide::Initiator) {
-            self.node.known_peers().remove(self.addr);
-        }
-    }
-}
