@@ -61,6 +61,6 @@ async fn broadcast_example() {
         1,
         random_nodes
             .iter()
-            .all(|rando| rando.node().stats().received().0 != 0)
+            .all(|rando| rando.node().stats().received().0 == 2)
     );
 }
